using System;
using System.ComponentModel;

namespace BrowsableAttribChange
{
public class MyPropertyDescriptor : PropertyDescriptor
{
    private PropertyDescriptor _propertyDescriptorOrg;
    public MyPropertyDescriptor(PropertyDescriptor field, Attribute[] attributes) :
        base(field.Name, attributes)
    {
        _propertyDescriptorOrg = field;
    }
    public override bool Equals(object obj)
    {
        MyPropertyDescriptor other = obj as MyPropertyDescriptor;
        return other != null && other._propertyDescriptorOrg.Equals(_propertyDescriptorOrg);
    }
    public override int GetHashCode() { return _propertyDescriptorOrg.GetHashCode(); }
    public override bool IsReadOnly { get { return _propertyDescriptorOrg.IsReadOnly; } }
    public override void ResetValue(object component) { _propertyDescriptorOrg.ResetValue(component); }
    public override bool CanResetValue(object component) { return _propertyDescriptorOrg.CanResetValue(component); }
    public override bool ShouldSerializeValue(object component) { return _propertyDescriptorOrg.ShouldSerializeValue(component); }
    public override Type ComponentType { get { return _propertyDescriptorOrg.ComponentType; } }
    public override Type PropertyType { get { return _propertyDescriptorOrg.PropertyType; } }
    public override object GetValue(object component) { return _propertyDescriptorOrg.GetValue(component); }
    public override void SetValue(object component, object value)
    {
        _propertyDescriptorOrg.SetValue(component, value);
        OnValueChanged(component, EventArgs.Empty);
    }
}

public class AttributesProxyTypeDescriptor : ICustomTypeDescriptor
{
    private object _target;
    public bool BrowsableAttributeAlwaysTrue { get; set; }
    public AttributesProxyTypeDescriptor(object target)
    {
        if (target == null) throw new ArgumentNullException("target");
        _target = target;
        BrowsableAttributeAlwaysTrue = false;
    }
    object ICustomTypeDescriptor.GetPropertyOwner(PropertyDescriptor pd)
    {
        return _target;
    }
    AttributeCollection ICustomTypeDescriptor.GetAttributes()
    {
        return TypeDescriptor.GetAttributes(_target, true);
    }
    string ICustomTypeDescriptor.GetClassName()
    {
        return TypeDescriptor.GetClassName(_target, true);
    }
    string ICustomTypeDescriptor.GetComponentName()
    {
        return TypeDescriptor.GetComponentName(_target, true);
    }
    TypeConverter ICustomTypeDescriptor.GetConverter()
    {
        return TypeDescriptor.GetConverter(_target, true);
    }
    EventDescriptor ICustomTypeDescriptor.GetDefaultEvent()
    {
        return TypeDescriptor.GetDefaultEvent(_target, true);
    }
    PropertyDescriptor ICustomTypeDescriptor.GetDefaultProperty()
    {
        return TypeDescriptor.GetDefaultProperty(_target, true);
    }
    object ICustomTypeDescriptor.GetEditor(Type editorBaseType)
    {
        return TypeDescriptor.GetEditor(_target, editorBaseType, true);
    }
    EventDescriptorCollection ICustomTypeDescriptor.GetEvents(Attribute[] attributes)
    {
        return TypeDescriptor.GetEvents(_target, attributes, true);
    }
    EventDescriptorCollection ICustomTypeDescriptor.GetEvents()
    {
        return TypeDescriptor.GetEvents(_target, true);
    }
    PropertyDescriptorCollection ICustomTypeDescriptor.GetProperties()
    {
        return ((ICustomTypeDescriptor)this).GetProperties(null);
    }
    PropertyDescriptorCollection ICustomTypeDescriptor.GetProperties(Attribute[] attributes)
    {
        PropertyDescriptorCollection props_tobereturned = new PropertyDescriptorCollection(null);
        if (BrowsableAttributeAlwaysTrue)
        {
            foreach (PropertyDescriptor prop in TypeDescriptor.GetProperties(_target, true))
            {

                props_tobereturned.Add(new MyPropertyDescriptor(prop, attributes));
            }
        }
        else
        {
            props_tobereturned = TypeDescriptor.GetProperties(_target, attributes, true);
        }
        return props_tobereturned;
    }

}
}